/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.ImageDiffer;

import java.awt.Color;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.gfxtest.framework.Log;

public class Configuration
{
    private boolean convertDiffImageToGrayscale = true;

    private int pixelValueThreshold = 10;
    private int differentPixelsThreshold = 10;

    private Color diffColorPixelValueUnderThreshold = new Color(0, 0, 255);
    private Color diffColorPerceptiblePixelDifference = new Color(255, 0, 0);
    private Color boundaryColor = new Color(0, 128, 255);

    private int boundarySize = 5;

    private File firstSourceDirectory = null;
    private File secondSourceDirectory = null;
    private File outputDirectory = null;

    private boolean createHtmlFiles = false;
    private boolean createHtmlResult = false;
    private boolean createXmlResult = false;
    private boolean writeDiffImages = false;
    private boolean writeStructDiffImages = false;

    private Log log = null;

    public Configuration()
    {
        this.log = new Log(this.getClass().getName(), true);
    }

    public Configuration(String args[])
    {
        this.log = new Log(this.getClass().getName(), true);
        this.log.logConfig("command line parameters: %d", Integer.valueOf(args.length));
        readCommandLineParameters(args);
        printParameters();
    }

    private void printParameters()
    {
        this.log.logConfig("first source directory:  %s", getAbsolutePath(this.firstSourceDirectory));
        this.log.logConfig("second source directory: %s", getAbsolutePath(this.secondSourceDirectory));
        this.log.logConfig("output directory:        %s", getAbsolutePath(this.outputDirectory));
        this.log.logConfig("HTML files creation:     %s", enabledFlag(this.isHtmlFilesCreationEnabled()));
        this.log.logConfig("HTML result creation:    %s", enabledFlag(this.isHtmlResultCreationEnabled()));
        this.log.logConfig("XML result creation:     %s", enabledFlag(this.isXmlResultCreationEnabled()));
        this.log.logConfig("Diff images creation:    %s", enabledFlag(this.isDiffImagesCreationEnabled()));
        this.log.logConfig("Struct. diff images:     %s", enabledFlag(this.isStructDiffImagesCreationEnabled()));
    }

    private String enabledFlag(boolean flag)
    {
        return flag ? "enabled" : "disabled";
    }

    private String getAbsolutePath(File directory)
    {
        return directory == null ? "not set" : directory.getAbsolutePath();
    }

    private void readCommandLineParameters(String args[])
    {
        Map<String, String> shortOptions = resolveAllOptions(args, "-");
        Map<String, String> longOptions = resolveAllOptions(args, "--");
        this.firstSourceDirectory = getPath(shortOptions, longOptions, "s1", "source1");
        this.secondSourceDirectory = getPath(shortOptions, longOptions, "s2", "source2");
        this.outputDirectory = getPath(shortOptions, longOptions, "o", "output");
        this.createHtmlFiles = getOption(shortOptions, longOptions, "t", "html");
        this.createHtmlResult = getOption(shortOptions, longOptions, "r", "html-results");
        this.createXmlResult = getOption(shortOptions, longOptions, "x", "xml-results");
        this.writeDiffImages = getOption(shortOptions, longOptions, "d", "diff-images");
        this.writeStructDiffImages = getOption(shortOptions, longOptions, "s", "struct-diff-images");
    }

    private boolean getOption(Map<String, String> shortOptions, Map<String, String> longOptions, String shortOptionName, String longOptionName)
    {
        return shortOptions.containsKey(shortOptionName) || longOptions.containsKey(longOptionName);
    }

    private File getPath(Map<String, String> shortOptions, Map<String, String> longOptions, String shortOptionName, String longOptionName)
    {
        String fileName = shortOptions.get(shortOptionName);
        if (fileName != null)
        {
            return new File(fileName);
        }
        fileName = longOptions.get(longOptionName);
        if (fileName != null)
        {
            return new File(fileName);
        }
        return null;
    }

    private Map<String, String> resolveAllOptions(String[] args, String prefix)
    {
        Map<String, String> options = new HashMap<String, String>();
        for (String arg : args)
        {
            String[] splittedArg = arg.split("=");
            if (splittedArg.length >= 1 && splittedArg[0].startsWith(prefix))
            {
                this.log.logConfig("found option %s", arg);
                String optionName = splittedArg[0].substring(prefix.length());
                String optionValue = splittedArg.length == 2 ? splittedArg[1] : "true";
                options.put(optionName, optionValue);
            }
        }
        return options;
    }

    public boolean isConvertDiffImageToGrayscale()
    {
        return this.convertDiffImageToGrayscale;
    }

    public void setConvertDiffImageToGrayscale(boolean convertDiffImageToGrayscale)
    {
        this.convertDiffImageToGrayscale = convertDiffImageToGrayscale;
    }

    public int getPixelValueThreshold()
    {
        return this.pixelValueThreshold;
    }

    public void setPixelValueThreshold(int pixelValueThreshold)
    {
        this.pixelValueThreshold = pixelValueThreshold;
    }

    public int getDifferentPixelsThreshold()
    {
        return this.differentPixelsThreshold;
    }

    public void setDifferentPixelsThreshold(int differentPixelsThreshold)
    {
        this.differentPixelsThreshold = differentPixelsThreshold;
    }

    public Color getDiffColorPixelValueUnderThreshold()
    {
        return this.diffColorPixelValueUnderThreshold;
    }

    public void setDiffColorPixelValueUnderThreshold(Color diffColorPixelValueUnderThreshold)
    {
        this.diffColorPixelValueUnderThreshold = diffColorPixelValueUnderThreshold;
    }

    public Color getDiffColorPerceptiblePixelDifference()
    {
        return this.diffColorPerceptiblePixelDifference;
    }

    public void setDiffColorPerceptiblePixelDifference(Color diffColorPerceptiblePixelDifference)
    {
        this.diffColorPerceptiblePixelDifference = diffColorPerceptiblePixelDifference;
    }

    public int getBoundarySize()
    {
        return this.boundarySize;
    }

    public void setBoundarySize(int boundarySize)
    {
        this.boundarySize = boundarySize;
    }

    public Color getBoundaryColor()
    {
        return this.boundaryColor;
    }

    public void setBoundaryColor(Color boundaryColor)
    {
        this.boundaryColor = boundaryColor;
    }

    public File getFirstSourceDirectory()
    {
        return this.firstSourceDirectory;
    }

    public File getSecondSourceDirectory()
    {
        return this.secondSourceDirectory;
    }

    public File getOutputDirectory()
    {
        return this.outputDirectory;
    }

    public boolean isHtmlFilesCreationEnabled()
    {
        return this.createHtmlFiles;
    }

    public boolean isHtmlResultCreationEnabled()
    {
        return this.createHtmlResult;
    }

    public boolean isXmlResultCreationEnabled()
    {
        return this.createXmlResult;
    }

    public boolean isDiffImagesCreationEnabled()
    {
        return this.writeDiffImages;
    }

    public boolean isStructDiffImagesCreationEnabled()
    {
        return this.writeStructDiffImages;
    }

}
