/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012, 2013, 2014  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

/**
 * This class contains static methods used by various CAG operations tests.
 * (CAG = Constructive Area Geometry)
 *
 * @author Pavel Tisnovsky
 */
public class CommonCAGOperations
{
    /**
     * Start angle for arcs.
     */
    private final static int ARC_START_ANGLE = 45;

    /**
     * Extend angle for arcs.
     */
    private final static int ARC_EXTEND_ANGLE = 270;

    /**
     * Start angle for chords.
     */
    private final static int CHORD_START_ANGLE = 45;

    /**
     * Extend angle for chords.
     */
    private final static int CHORD_EXTEND_ANGLE = 270;

    /**
     * Start angle for pies.
     */
    private final static int PIE_START_ANGLE = 45;

    /**
     * Extend angle for pies.
     */
    private final static int PIE_EXTEND_ANGLE = 270;

    /**
     * Radius of rounded
     */
    private final static int ROUND_RECTANGLE_RADIUS = 100;

    /**
     * Compute radius of circle from the position of its center point in an
     * image size (width and height).
     * 
     * @param xc
     *            x-coordinate of the center of the test image.
     * @param y
     *            y-coordinate of the center of the test image.
     * @return radius of circle
     */
    private static int computeRadiusForOverlappingCircle(int xc, int yc)
    {
        return ((xc > yc ? yc : xc) << 1) / 3;
    }

    /**
     * Compute radius of circle from the position of its center point in an
     * image size (width and height).
     * 
     * @param xc
     *            x-coordinate of the center of the test image.
     * @param y
     *            y-coordinate of the center of the test image.
     * @return radius of circle
     */
    private static int computeRadiusForTouchingCircle(int xc, int yc)
    {
        return (xc > yc ? yc : xc) >> 1;
    }

    /**
     * Compute size of the rectangle from the position of center point in an
     * image.
     * 
     * @param xc
     *            x-coordinate of the center of the test image.
     * @param y
     *            y-coordinate of the center of the test image.
     * @return size of the rectangle
     */
    private static int computeSizeForOverlappingRectangle(int xc, int yc)
    {
        return (xc > yc ? yc : xc) >> 1;
    }

    /**
     * Compute size of the rectangle or circle from the position of center point
     * in an image.
     * 
     * @param xc
     *            x-coordinate of the center of the test image.
     * @param y
     *            y-coordinate of the center of the test image.
     * @return size of the circle or rectangle
     */
    private static int computeSizeOfCircleOrRectangle(int xc, int yc)
    {
        return (xc > yc ? yc : xc) >> 1;
    }

    /**
     * Compute radius of bigger circle from the position of center point in an image.
     * 
     * @param xc
     *            x-coordinate of the center of the test image.
     * @param y
     *            y-coordinate of the center of the test image.
     * @return radius of circle
     */
    private static int computeRadiusForBiggerCircle(int xc, int yc)
    {
        return ((xc > yc ? yc : xc) << 1) / 3;
    }

    /**
     * Compute radius of smaller circle from the position of center point in an image.
     * 
     * @param xc
     *            x-coordinate of the center of the test image.
     * @param y
     *            y-coordinate of the center of the test image.
     * @return radius of circle
     */
    private static int computeRadiusForSmallerCircle(int xc, int yc)
    {
        return ((xc > yc ? yc : xc) << 1) / 4;
    }

    /**
     * Create circular area i.e. area consisting of just one circle.
     * 
     * @param xc
     *            the X coordinate of the center of circle
     * @param yc
     *            the Y coordinate of the center of circle
     * @param radius
     *            radius of circle
     * @return newly created area containing one circle
     */
    private static Area createCircularArea(int xc, int yc, int radius)
    {
        Ellipse2D circle = new Ellipse2D.Double(xc - radius, yc - radius, radius << 1, radius << 1);
        return new Area(circle);
    }

    /**
     * Create chord area i.e. area consisting of just one chord.
     * 
     * @param xc
     *            the X coordinate of the center of chord
     * @param yc
     *            the Y coordinate of the center of chord
     * @param radius
     *            radius of chord
     * @return newly created area containing one chord
     */
    private static Area createChordArea(int xc, int yc, int radius)
    {
        Arc2D chord = new Arc2D.Double(xc - radius, yc - radius, radius << 1, radius << 1, CHORD_START_ANGLE, CHORD_EXTEND_ANGLE, Arc2D.CHORD);
        return new Area(chord);
    }

    /**
     * Create pie area i.e. area consisting of just one pie.
     * 
     * @param xc
     *            the X coordinate of the center of pie
     * @param yc
     *            the Y coordinate of the center of pie
     * @param radius
     *            radius of pie
     * @return newly created area containing one pie
     */
    private static Area createPieArea(int xc, int yc, int radius)
    {
        Arc2D pie = new Arc2D.Double(xc - radius, yc - radius, radius << 1, radius << 1, PIE_START_ANGLE, PIE_EXTEND_ANGLE, Arc2D.PIE);
        return new Area(pie);
    }

    /**
     * Create rectangular area i.e. area consisting of just one rectangle.
     * 
     * @param xc
     *            the X coordinate of the center of rectangle
     * @param yc
     *            the Y coordinate of the center of rectangle
     * @param size
     *            of rectangle
     * @return newly created area containing one rectangle
     */
    private static Area createRectangularArea(int xc, int yc, int size)
    {
        Rectangle2D rectangle = new Rectangle2D.Double(xc - size, yc - size, size << 1, size << 1);
        return new Area(rectangle);
    }

    /**
     * Create rounded rectangular area i.e. area consisting of just one rectangle.
     * 
     * @param xc
     *            the X coordinate of the center of rectangle
     * @param yc
     *            the Y coordinate of the center of rectangle
     * @param size
     *            of rectangle
     * @return newly created area containing one rectangle
     */
    private static Area createRoundedRectangularArea(int xc, int yc, int size)
    {
        RoundRectangle2D rectangle = new RoundRectangle2D.Double(xc - size, yc - size, size << 1, size << 1, ROUND_RECTANGLE_RADIUS, ROUND_RECTANGLE_RADIUS);
        return new Area(rectangle);
    }

    /**
     * Create area composed of only one circle. This circle is placed on the
     * left side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static  Area createFirstOverlappingCircularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForOverlappingCircle(xc, yc);
        // move circle to the left side of image
        int x1 = xc - 3 * (radius >> 2);
        return createCircularArea(x1, yc, radius);
    }

    /**
     * Create area composed of only one circle. This circle is placed on the
     * right side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static Area createSecondOverlappingCircularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForOverlappingCircle(xc, yc);
        // move circle to the right side of image
        int x2 = xc + 3 * (radius >> 2);
        return createCircularArea(x2, yc, radius);
    }

    /**
     * Create area composed of only one circle. This circle is placed on the
     * left side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static  Area createFirstTouchingCircularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForTouchingCircle(xc, yc);
        // move circle to the left side of image
        int x1 = xc - radius;
        return createCircularArea(x1, yc, radius);
    }

    /**
     * Create area composed of only one chord. This chord is placed on the
     * left side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one chord
     */
    private static  Area createFirstOverlappingChordArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForOverlappingCircle(xc, yc);
        // move circle to the left side of image
        int x1 = xc - 3 * (radius >> 2);
        return createChordArea(x1, yc, radius);
    }

    /**
     * Create area composed of only one chord. This chord is placed on the
     * right side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one chord
     */
    private static Area createSecondOverlappingChordArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForOverlappingCircle(xc, yc);
        // move circle to the right side of image
        int x2 = xc + 3 * (radius >> 2);
        return createChordArea(x2, yc, radius);
    }

    /**
     * Create area composed of only one circle. This circle is placed on the
     * right side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static Area createSecondTouchingCircularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForTouchingCircle(xc, yc);
        // move circle to the right side of image
        int x2 = xc + radius;
        return createCircularArea(x2, yc, radius);
    }

    /**
     * Create area composed of only one rectangle. This rectangle is placed on
     * the upper left side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    private static Area createFirstOverlappingRectangularArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the rectangle
        int size = computeSizeForOverlappingRectangle(xc, yc);
        int x = xc - 3 * (size >> 2);
        // it's better to align rectangle sides with the grid
        x = x + (x % image.getGrid());
        int y = yc - (size >> 1);
        return createRectangularArea(x, y, size);
    }

    /**
     * Create area composed of only one rounded rectangle. This rectangle is placed on
     * the upper left side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    private static Area createFirstOverlappingRoundedRectangularArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the rectangle
        int size = computeSizeForOverlappingRectangle(xc, yc);
        int x = xc - 3 * (size >> 2);
        // it's better to align rectangle sides with the grid
        x = x + (x % image.getGrid());
        int y = yc - (size >> 1);
        return createRoundedRectangularArea(x, y, size);
    }

    /**
     * Create area composed of only one rectangle. This rectangle is placed on
     * the lower right side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    private static Area createSecondOverlappingRectangularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the rectangle
        int size = computeSizeForOverlappingRectangle(xc, yc);
        int x = xc + 3 * (size >> 2);
        // it's better to align rectangle sides with the grid
        x = x - (x % image.getGrid());
        int y = yc + (size >> 1);
        return createRectangularArea(x, y, size);
    }

    /**
     * Create area composed of only one round rectangle. This rectangle is placed on
     * the lower right side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    private static Area createSecondOverlappingRoundedRectangularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the rectangle
        int size = computeSizeForOverlappingRectangle(xc, yc);
        int x = xc + 3 * (size >> 2);
        // it's better to align rectangle sides with the grid
        x = x - (x % image.getGrid());
        int y = yc + (size >> 1);
        return createRoundedRectangularArea(x, y, size);
    }

    /**
     * Create area composed of only one rectangle. This rectangle is placed on
     * the center of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    private static Area createCenteredRectangularArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the rectangle
        int size = 3 * computeSizeOfCircleOrRectangle(xc, yc) / 2 - 10;
        // it's better to align rectangle sides with the grid
        return createRectangularArea(xc, yc, size);
    }

    /**
     * Create area composed of only one circle. This circle is placed on
     * the center of the image and its radius is bigger than the length
     * of rectangle side(s).
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static Area createBiggerCenteredCircularArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the circle
        int size = computeSizeOfCircleOrRectangle(xc, yc) * 5 / 3;
        return createCircularArea(xc, yc, size);
    }

    /**
     * Create area composed of only one chord. This chord is placed on
     * the center of the image and its radius is bigger than the length
     * of rectangle side(s).
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one chord
     */
    private static Area createBiggerCenteredChordArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the circle
        int size = computeSizeOfCircleOrRectangle(xc, yc) * 5 / 3;
        return createChordArea(xc, yc, size);
    }

    /**
     * Create area composed of only one pie. This pie is placed on
     * the center of the image and its radius is bigger than the length
     * of rectangle side(s).
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one pie
     */
    private static Area createBiggerCenteredPieArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the circle
        int size = computeSizeOfCircleOrRectangle(xc, yc) * 5 / 3;
        return createPieArea(xc, yc, size);
    }

    /**
     * Create area composed of only one circle. This circle is placed on
     * the center of the image and its radius is smaller than the length
     * of rectangle side(s).
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static Area createSmallerCenteredCircularArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the circle
        int size = computeSizeOfCircleOrRectangle(xc, yc);
        return createCircularArea(xc, yc, size);
    }

    /**
     * Create area composed of only one chord. This chord is placed on
     * the center of the image and its radius is smaller than the length
     * of rectangle side(s).
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static Area createSmallerCenteredChordArea(TestImage image)
    {
        // compute center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        // compute size of the circle
        int size = computeSizeOfCircleOrRectangle(xc, yc);
        return createChordArea(xc, yc, size);
    }

    /**
     * Create area composed of only one circle. This circle is placed on the
     * left side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static Area createBiggerConcentricCircularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForBiggerCircle(xc, yc);
        return createCircularArea(xc, yc, radius);
    }

    /**
     * Create area composed of only one circle. This circle is placed on the
     * right side of the image.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one circle
     */
    private static Area createSmallerConcentricCircularArea(TestImage image)
    {
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        int radius = computeRadiusForSmallerCircle(xc, yc);
        return createCircularArea(xc, yc, radius);
    }

    /**
     * Create new area composed from two overlapping circles using union
     * operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoOverlappingCirclesUsingUnionOperator(TestImage image)
    {
        Area circle1Area = createFirstOverlappingCircularArea(image);
        Area circle2Area = createSecondOverlappingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.add(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two overlapping circles using subtract
     * operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoOverlappingCirclesUsingSubtractOperator(TestImage image)
    {
        Area circle1Area = createFirstOverlappingCircularArea(image);
        Area circle2Area = createSecondOverlappingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.subtract(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two overlapping circles using subtract
     * operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoOverlappingCirclesUsingInverseSubtractOperator(TestImage image)
    {
        Area circle1Area = createFirstOverlappingCircularArea(image);
        Area circle2Area = createSecondOverlappingCircularArea(image);
        Area area = new Area();
        area.add(circle2Area);
        area.subtract(circle1Area);
        return area;
    }

    /**
     * Create new area composed from two overlapping circles using intersect
     * operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoOverlappingCirclesUsingIntersectOperator(TestImage image)
    {
        Area circle1Area = createFirstOverlappingCircularArea(image);
        Area circle2Area = createSecondOverlappingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.intersect(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two overlapping circles using XOR operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoOverlappingCirclesUsingXorOperator(TestImage image)
    {
        Area circle1Area = createFirstOverlappingCircularArea(image);
        Area circle2Area = createSecondOverlappingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.exclusiveOr(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two rectangles using union operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRectanglesUsingUnionOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.add(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from two rectangles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRectanglesUsingSubtractOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.subtract(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from two rectangles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRectanglesUsingInverseSubtractOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle2Area);
        area.subtract(rectangle1Area);
        return area;
    }

    /**
     * Create new area composed from two rectangles using intersect operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRectanglesUsingIntersectOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.intersect(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from two rectangles using XOR operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRectanglesUsingXorOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle2Area);
        area.exclusiveOr(rectangle1Area);
        return area;
    }

    /**
     * Create new area composed from two round rectangles using union operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRoundRectanglesUsingUnionOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRoundedRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.add(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from two round rectangles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRoundRectanglesUsingSubtractOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRoundedRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.subtract(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from two round rectangles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRoundRectanglesUsingInverseSubtractOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRoundedRectangularArea(image);
        Area area = new Area();
        area.add(rectangle2Area);
        area.subtract(rectangle1Area);
        return area;
    }

    /**
     * Create new area composed from two round rectangles using intersect operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRoundRectanglesUsingIntersectOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRoundedRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.intersect(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from two round rectangles using XOR operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromTwoOverlappingRoundRectanglesUsingXorOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRoundedRectangularArea(image);
        Area area = new Area();
        area.add(rectangle2Area);
        area.exclusiveOr(rectangle1Area);
        return area;
    }

    /**
     * Create new area composed from rounded rectangle and rectangle using union operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromRoundRectangleAndRectangleUsingUnionOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.add(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from rounded rectangle and rectangle using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromRoundRectangleAndRectangleUsingSubtractOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.subtract(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from rounded rectangle and rectangle using inverse subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromRoundRectangleAndRectangleUsingInverseSubtractOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle2Area);
        area.subtract(rectangle1Area);
        return area;
    }

    /**
     * Create new area composed from rounded rectangle and rectangle using intersect operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromRoundRectangleAndRectangleUsingIntersectOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.intersect(rectangle2Area);
        return area;
    }

    /**
     * Create new area composed from rounded rectangle and rectangle using exclusive or operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two rectangles
     */
    public static Area createAreaFromRoundRectangleAndRectangleUsingXorOperator(TestImage image)
    {
        Area rectangle1Area = createFirstOverlappingRoundedRectangularArea(image);
        Area rectangle2Area = createSecondOverlappingRectangularArea(image);
        Area area = new Area();
        area.add(rectangle1Area);
        area.exclusiveOr(rectangle2Area);
        return area;
    }

    /**
     * Create new area constructed from bigger circle and rectangle using union
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerCircleAndRectangleUsingUnionOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredCircularArea(image));
        area.add(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger circle and rectangle using intersect
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerCircleAndRectangleUsingIntersectOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredCircularArea(image));
        area.intersect(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger circle and rectangle using subtract
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerCircleAndRectangleUsingSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredCircularArea(image));
        area.subtract(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger circle and rectangle using
     * inverse subtract operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerCircleAndRectangleUsingInverseSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.subtract(createBiggerCenteredCircularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger circle and rectangle using Xor
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerCircleAndRectangleUsingXorOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredCircularArea(image));
        area.exclusiveOr(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger chord and rectangle using union
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerChordAndRectangleUsingUnionOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredChordArea(image));
        area.add(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger chord and rectangle using intersect
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerChordAndRectangleUsingIntersectOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredChordArea(image));
        area.intersect(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger chord and rectangle using subtract
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerChordAndRectangleUsingSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredChordArea(image));
        area.subtract(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger chord and rectangle using inverse subtract
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerChordAndRectangleUsingInverseSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.subtract(createBiggerCenteredChordArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger chord and rectangle using xor
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerChordAndRectangleUsingXorOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.exclusiveOr(createBiggerCenteredChordArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger pie and rectangle using union
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerPieAndRectangleUsingUnionOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredPieArea(image));
        area.add(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger pie and rectangle using intersect
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerPieAndRectangleUsingIntersectOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredPieArea(image));
        area.intersect(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger pie and rectangle using subtract
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerPieAndRectangleUsingSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createBiggerCenteredPieArea(image));
        area.subtract(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger pie and rectangle using inverse subtract
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerPieAndRectangleUsingInverseSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.subtract(createBiggerCenteredPieArea(image));
        return area;
    }

    /**
     * Create new area constructed from bigger pie and rectangle using xor
     * operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area
     */
    public static Area createAreaFromBiggerPieAndRectangleUsingXorOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.exclusiveOr(createBiggerCenteredPieArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller circle placed inside a rectangle
     * using union operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerCircleInsideRectangleUsingUnionOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.add(createSmallerCenteredCircularArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller circle placed inside a rectangle
     * using subtract operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerCircleInsideRectangleUsingSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.subtract(createSmallerCenteredCircularArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller circle placed inside a rectangle
     * using inverse subtract operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerCircleInsideRectangleUsingInverseSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createSmallerCenteredCircularArea(image));
        area.subtract(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller circle placed inside a rectangle
     * using intersect operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerCircleInsideRectangleUsingIntersectOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createSmallerCenteredCircularArea(image));
        area.intersect(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller circle placed inside a rectangle
     * using XOR operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerCircleInsideRectangleUsingXorOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createSmallerCenteredCircularArea(image));
        area.exclusiveOr(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller chord placed inside a rectangle
     * using union operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerChordInsideRectangleUsingUnionOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.add(createSmallerCenteredChordArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller chord placed inside a rectangle
     * using subtract operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerChordInsideRectangleUsingSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createCenteredRectangularArea(image));
        area.subtract(createSmallerCenteredChordArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller chord placed inside a rectangle
     * using inverse subtract operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerChordInsideRectangleUsingInverseSubtractOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createSmallerCenteredChordArea(image));
        area.subtract(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller chord placed inside a rectangle
     * using intersect operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerChordInsideRectangleUsingIntersectOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createSmallerCenteredChordArea(image));
        area.intersect(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area constructed from smaller chord placed inside a rectangle
     * using XOR operation.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return newly created area containing one rectangle
     */
    public static Area createAreaFromSmallerChordInsideRectangleUsingXorOperator(TestImage image)
    {
        Area area = new Area();
        area.add(createSmallerCenteredChordArea(image));
        area.exclusiveOr(createCenteredRectangularArea(image));
        return area;
    }

    /**
     * Create new area composed from two concentric circles using union operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoConcentricCirclesUsingUnionOperator(TestImage image)
    {
        Area circle1Area = createBiggerConcentricCircularArea(image);
        Area circle2Area = createSmallerConcentricCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.add(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two concentric circles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoConcentricCirclesUsingSubtractOperator(TestImage image)
    {
        Area circle1Area = createBiggerConcentricCircularArea(image);
        Area circle2Area = createSmallerConcentricCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.subtract(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two concentric circles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoConcentricCirclesUsingInverseSubtractOperator(TestImage image)
    {
        Area circle1Area = createBiggerConcentricCircularArea(image);
        Area circle2Area = createSmallerConcentricCircularArea(image);
        Area area = new Area();
        area.add(circle2Area);
        area.subtract(circle1Area);
        return area;
    }

    /**
     * Create new area composed from two concentric circles using intersect operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoConcentricCirclesUsingIntersectOperator(TestImage image)
    {
        Area circle1Area = createBiggerConcentricCircularArea(image);
        Area circle2Area = createSmallerConcentricCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.intersect(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two concentric circles using XOR operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoConcentricCirclesUsingXorOperator(TestImage image)
    {
        Area circle1Area = createBiggerConcentricCircularArea(image);
        Area circle2Area = createSmallerConcentricCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.exclusiveOr(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two touching circles using union operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoTouchingCirclesUsingUnionOperator(TestImage image)
    {
        Area circle1Area = createFirstTouchingCircularArea(image);
        Area circle2Area = createSecondTouchingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.add(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two touching circles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoTouchingCirclesUsingSubtractOperator(TestImage image)
    {
        Area circle1Area = createFirstTouchingCircularArea(image);
        Area circle2Area = createSecondTouchingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.subtract(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two touching circles using subtract operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoTouchingCirclesUsingInverseSubtractOperator(TestImage image)
    {
        Area circle1Area = createFirstTouchingCircularArea(image);
        Area circle2Area = createSecondTouchingCircularArea(image);
        Area area = new Area();
        area.add(circle2Area);
        area.subtract(circle1Area);
        return area;
    }

    /**
     * Create new area composed from two touching circles using intersect operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoTouchingCirclesUsingIntersectOperator(TestImage image)
    {
        Area circle1Area = createFirstTouchingCircularArea(image);
        Area circle2Area = createSecondTouchingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.intersect(circle2Area);
        return area;
    }

    /**
     * Create new area composed from two touching circles using XOR operator.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed of two circles
     */
    public static Area createAreaFromTwoTouchingCirclesUsingXorOperator(TestImage image)
    {
        Area circle1Area = createFirstTouchingCircularArea(image);
        Area circle2Area = createSecondTouchingCircularArea(image);
        Area area = new Area();
        area.add(circle1Area);
        area.exclusiveOr(circle2Area);
        return area;
    }

    /**
     * Create new area composed from a circle and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a circle and a rectangle
     */
    public static Area createAreaFromOverlappingCircleAndRectangleUsingUnionOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingCircularArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.add(rectangularArea);
        return area;
    }

    /**
     * Create new area composed from a circle and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a circle and a rectangle
     */
    public static Area createAreaFromOverlappingCircleAndRectangleUsingSubtractOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingCircularArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.subtract(rectangularArea);
        return area;
    }

    /**
     * Create new area composed from a circle and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a circle and a rectangle
     */
    public static Area createAreaFromOverlappingCircleAndRectangleUsingInverseSubtractOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingCircularArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(rectangularArea);
        area.subtract(circleArea);
        return area;
    }

    /**
     * Create new area composed from a circle and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a circle and a rectangle
     */
    public static Area createAreaFromOverlappingCircleAndRectangleUsingIntersectOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingCircularArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.intersect(rectangularArea);
        return area;
    }

    /**
     * Create new area composed from a circle and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a circle and a rectangle
     */
    public static Area createAreaFromOverlappingCircleAndRectangleUsingXorOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingCircularArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.exclusiveOr(rectangularArea);
        return area;
    }

    /**
     * Create new area composed from a chord and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a chord and a rectangle
     */
    public static Area createAreaFromOverlappingChordAndRectangleUsingUnionOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingChordArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.add(rectangularArea);
        return area;
    }

    /**
     * Create new area composed from a chord and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a chord and a rectangle
     */
    public static Area createAreaFromOverlappingChordAndRectangleUsingSubtractOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingChordArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.subtract(rectangularArea);
        return area;
    }

    /**
     * Create new area composed from a chord and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a chord and a rectangle
     */
    public static Area createAreaFromOverlappingChordAndRectangleUsingInverseSubtractOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingChordArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(rectangularArea);
        area.subtract(circleArea);
        return area;
    }

    /**
     * Create new area composed from a chord and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a chord and a rectangle
     */
    public static Area createAreaFromOverlappingChordAndRectangleUsingIntersectOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingChordArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.intersect(rectangularArea);
        return area;
    }

    /**
     * Create new area composed from a chord and a rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return new area composed from a chord and a rectangle
     */
    public static Area createAreaFromOverlappingChordAndRectangleUsingXorOperator(TestImage image) {
        Area area = new Area();
        Area circleArea = createFirstOverlappingChordArea(image);
        Area rectangularArea = createSecondOverlappingRectangularArea(image);
        area.add(circleArea);
        area.exclusiveOr(rectangularArea);
        return area;
    }

}
