/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.gfxtest.testsuites;

import java.awt.Graphics2D;



import org.gfxtest.framework.CommonClippingOperations;
import org.gfxtest.framework.CommonRenderingStyles;
import org.gfxtest.framework.CommonShapesRenderer;
import org.gfxtest.framework.GfxTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;

/**
 * This test check if clipping is working correctly for an empty, filled
 * and textured circle which is clipped by an ellipse shape.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@Transformation(Transformations.NONE)
@GraphicsPrimitive(GraphicsPrimitives.CIRCLE)
@Zoom(1)
public class ClippingCircleByEllipseShape extends GfxTest
{
    /**
     * Prepare canvas for the rendering.
     *
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void basicSetupForRendering(TestImage image, Graphics2D graphics2d)
    {
        // create clip area which is constructed from the ellipse shape
        CommonClippingOperations.createClipUsingEllipseShape(image, graphics2d);
    }

    /**
     * Draw empty circle clipped by an ellipse shape.
     *
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawEmptyCircleClippedByEllipseShape(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the area
        CommonShapesRenderer.drawCircle(image, graphics2d);
    }

    /**
     * Draw filled circle clipped by an ellipse shape.
     *
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawFilledCircleClippedByEllipseShape(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // fill the area
        CommonShapesRenderer.drawFilledCircle(image, graphics2d);
    }

    /**
     * Draw circle clipped by an ellipse shape. Circle is drawn using alpha
     * paint with black color and selected transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @param transparency
     *            selected transparency (0..100 percent)
     */
    private void drawCircleClippedByEllipseShapeAlphaPaintBlack(TestImage image, Graphics2D graphics2d, int transparency)
    {
        // render clip ellipse
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set fill color
        CommonRenderingStyles.setTransparentFillBlackColor(graphics2d, transparency);
        // create clip area
        CommonClippingOperations.createClipUsingEllipseShape(image, graphics2d);
        // fill the shape
        CommonShapesRenderer.drawFilledCircle(image, graphics2d);
    }

    /**
     * Draw circle clipped by an ellipse shape. Circle is drawn using alpha
     * paint with red color and selected transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @param transparency
     *            selected transparency (0..100 percent)
     */
    private void drawCircleClippedByEllipseShapeAlphaPaintRed(TestImage image, Graphics2D graphics2d, int transparency)
    {
        // render clip ellipse
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set fill color
        CommonRenderingStyles.setTransparentFillRedColor(graphics2d, transparency);
        // create clip area
        CommonClippingOperations.createClipUsingEllipseShape(image, graphics2d);
        // fill the shape
        CommonShapesRenderer.drawFilledCircle(image, graphics2d);
    }

    /**
     * Draw circle clipped by an ellipse shape. Circle is drawn using alpha
     * paint with green color and selected transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @param transparency
     *            selected transparency (0..100 percent)
     */
    private void drawCircleClippedByEllipseShapeAlphaPaintGreen(TestImage image, Graphics2D graphics2d, int transparency)
    {
        // render clip ellipse
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set fill color
        CommonRenderingStyles.setTransparentFillGreenColor(graphics2d, transparency);
        // create clip area
        CommonClippingOperations.createClipUsingEllipseShape(image, graphics2d);
        // fill the shape
        CommonShapesRenderer.drawFilledCircle(image, graphics2d);
    }

    /**
     * Draw circle clipped by an ellipse shape. Circle is drawn using alpha
     * paint with blue color and selected transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @param transparency
     *            selected transparency (0..100 percent)
     */
    private void drawCircleClippedByEllipseShapeAlphaPaintBlue(TestImage image, Graphics2D graphics2d, int transparency)
    {
        // render clip ellipse
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set fill color
        CommonRenderingStyles.setTransparentFillBlueColor(graphics2d, transparency);
        // create clip area
        CommonClippingOperations.createClipUsingEllipseShape(image, graphics2d);
        // fill the shape
        CommonShapesRenderer.drawFilledCircle(image, graphics2d);
    }

    /**
     * Draw circle clipped by an ellipse shape. Circle is drawn using alpha
     * paint with white color and selected transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @param transparency
     *            selected transparency (0..100 percent)
     */
    private void drawCircleClippedByEllipseShapeAlphaPaintWhite(TestImage image, Graphics2D graphics2d, int transparency)
    {
        // render clip ellipse
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set fill color
        CommonRenderingStyles.setTransparentFillWhiteColor(graphics2d, transparency);
        // create clip area
        CommonClippingOperations.createClipUsingEllipseShape(image, graphics2d);
        // fill the shape
        CommonShapesRenderer.drawFilledCircle(image, graphics2d);
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using stroke paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set clip region and draw the circle
        drawEmptyCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using wide stroke paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the circle
        drawEmptyCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using extra wide stroke paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeExtraWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the circle
        drawEmptyCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using zero wide stroke paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeZeroWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the circle
        drawEmptyCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using color paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeColorPaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set fill color
        CommonRenderingStyles.setFillColor(graphics2d);
        // create clip shape
        CommonClippingOperations.createClipUsingEllipseShape(image, graphics2d);
        // fill the shape
        CommonShapesRenderer.drawFilledCircle(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with black color at 0% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlack000(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 0% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlack(image, graphics2d, 0);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with black color at 25% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlack025(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 25% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlack(image, graphics2d, 25);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with black color at 50% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlack050(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 50% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlack(image, graphics2d, 50);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with black color at 75% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlack075(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 75% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlack(image, graphics2d, 75);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with black color at 100% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlack100(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 100% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlack(image, graphics2d, 100);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with red color at 0% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintRed000(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 0% transparency
        drawCircleClippedByEllipseShapeAlphaPaintRed(image, graphics2d, 0);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with red color at 25% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintRed025(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 25% transparency
        drawCircleClippedByEllipseShapeAlphaPaintRed(image, graphics2d, 25);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with red color at 50% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintRed050(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 50% transparency
        drawCircleClippedByEllipseShapeAlphaPaintRed(image, graphics2d, 50);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with red color at 75% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintRed075(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 75% transparency
        drawCircleClippedByEllipseShapeAlphaPaintRed(image, graphics2d, 75);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with red color at 100% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintRed100(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 100% transparency
        drawCircleClippedByEllipseShapeAlphaPaintRed(image, graphics2d, 100);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with green color at 0% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintGreen000(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 0% transparency
        drawCircleClippedByEllipseShapeAlphaPaintGreen(image, graphics2d, 0);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with green color at 25% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintGreen025(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 25% transparency
        drawCircleClippedByEllipseShapeAlphaPaintGreen(image, graphics2d, 25);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with green color at 50% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintGreen050(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 50% transparency
        drawCircleClippedByEllipseShapeAlphaPaintGreen(image, graphics2d, 50);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with green color at 75% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintGreen075(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 75% transparency
        drawCircleClippedByEllipseShapeAlphaPaintGreen(image, graphics2d, 75);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with green color at 100% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintGreen100(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 100% transparency
        drawCircleClippedByEllipseShapeAlphaPaintGreen(image, graphics2d, 100);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with blue color at 0% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlue000(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 0% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlue(image, graphics2d, 0);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with blue color at 25% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlue025(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 25% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlue(image, graphics2d, 25);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with blue color at 50% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlue050(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 50% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlue(image, graphics2d, 50);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with blue color at 75% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlue075(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 75% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlue(image, graphics2d, 75);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with blue color at 100% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintBlue100(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 100% transparency
        drawCircleClippedByEllipseShapeAlphaPaintBlue(image, graphics2d, 100);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with white color at 0% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintWhite000(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 0% transparency
        drawCircleClippedByEllipseShapeAlphaPaintWhite(image, graphics2d, 0);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with white color at 25% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintWhite025(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 25% transparency
        drawCircleClippedByEllipseShapeAlphaPaintWhite(image, graphics2d, 25);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with white color at 50% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintWhite050(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 50% transparency
        drawCircleClippedByEllipseShapeAlphaPaintWhite(image, graphics2d, 50);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with white color at 75% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintWhite075(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 75% transparency
        drawCircleClippedByEllipseShapeAlphaPaintWhite(image, graphics2d, 75);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using alpha paint with white color at 100% transparency.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeAlphaPaintWhite100(TestImage image, Graphics2D graphics2d)
    {
        // draw circle clipped by ellipse shape using alpha paint with 100% transparency
        drawCircleClippedByEllipseShapeAlphaPaintWhite(image, graphics2d, 100);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using horizontal gradient paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeHorizontalGradientPaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set horizontal gradient paint
        CommonRenderingStyles.setHorizontalGradientFill(image, graphics2d);
        // set clip region and draw the circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using vertical gradient paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeVerticalGradientPaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set vertical gradient paint
        CommonRenderingStyles.setVerticalGradientFill(image, graphics2d);
        // set clip region and draw the circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using diagonal gradient paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeDiagonalGradientPaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set diagonal gradient paint
        CommonRenderingStyles.setDiagonalGradientFill(image, graphics2d);
        // set clip region and draw the circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using radial gradient paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeRadialGradientPaint(TestImage image, Graphics2D graphics2d)
    {
        // render an ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set radial gradient paint
        CommonRenderingStyles.setRadialGradientFill(image, graphics2d);
        // set clip region and draw the circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeCheckerTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingCheckerTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeDiagonalCheckerTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingDiagonalCheckerTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeGridTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingGridTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeDiagonalGridTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingDiagonalGridTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeColorDotsTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingColorDotsTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeHorizontalStripesTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingHorizontalStripesTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeVerticalStripesTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingVerticalStripesTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeDiagonalStripesTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingDiagonalStripesTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeHorizontalColorStripesTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingHorizontalColorStripesTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeVerticalColorStripesTexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingVerticalColorStripesTexture(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeRGB1TexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingRGBTexture1(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeRGB2TexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingRGBTexture2(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if circle shape could be clipped by an ellipse shape. Circle is
     * rendered using texture paint.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCircleByEllipseShapeRGB3TexturePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip ellipse which is used as a base for clip shape
        CommonClippingOperations.renderClipEllipse(image, graphics2d);
        // set the texture
        CommonRenderingStyles.setTextureFillUsingRGBTexture3(image, graphics2d);
        // set clip region and render filled circle
        drawFilledCircleClippedByEllipseShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            graphics test configuration
     */
    public static void main(String[] args)
    {
        new ClippingCircleByEllipseShape().runTestSuite(args);
    }
}
